package ru.dbotthepony.kstarbound.defs.actor.behavior

import com.google.gson.JsonNull
import com.google.gson.TypeAdapter
import com.google.gson.annotations.JsonAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.gson.contains
import ru.dbotthepony.kommons.gson.value
import ru.dbotthepony.kstarbound.json.popObject
import ru.dbotthepony.kstarbound.util.valueOf
import ru.dbotthepony.kstarbound.world.entities.behavior.NodeParameterType

@JsonAdapter(NodeParameter.Adapter::class)
data class NodeParameter(val type: NodeParameterType, val value: NodeParameterValue) {
	override fun toString(): String {
		return "[$type as $value]"
	}

	class Adapter : TypeAdapter<NodeParameter>() {
		override fun write(out: JsonWriter, value: NodeParameter) {
			out.beginObject()

			out.name("type")
			out.value(value.type.jsonName)

			if (value.value.key != null) {
				out.name("key")
				out.value(value.value.key)
			} else {
				out.name("value")
				out.value(value.value.value)
			}

			out.endObject()
		}

		override fun read(`in`: JsonReader): NodeParameter {
			val json = `in`.popObject()
			val type = NodeParameterType.entries.valueOf(json["type"].asString)

			if ("key" in json) {
				return NodeParameter(type, NodeParameterValue(json["key"].asString, null))
			} else {
				return NodeParameter(type, NodeParameterValue(null, json["value"] ?: JsonNull.INSTANCE))
			}
		}
	}
}
