package ru.dbotthepony.kstarbound.defs.world

import com.google.gson.TypeAdapter
import com.google.gson.annotations.JsonAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.gson.contains
import ru.dbotthepony.kstarbound.json.popObject
import ru.dbotthepony.kstarbound.json.stream
import ru.dbotthepony.kstarbound.util.valueOf
import java.util.EnumSet
import java.util.stream.Collectors

@JsonAdapter(SpawnParameters.Adapter::class)
data class SpawnParameters(val areas: Set<SpawnArea> = SpawnArea.ALL, val region: SpawnRegion = SpawnRegion.ALL, val time: SpawnTime = SpawnTime.ALL) {
	fun isCompatible(other: SpawnParameters): Boolean {
		return region.isCompatible(other.region) && time.isCompatible(other.time) && areas.any { it in other.areas }
	}

	class Adapter : TypeAdapter<SpawnParameters>() {
		override fun write(out: JsonWriter, value: SpawnParameters) {
			out.beginObject()
			out.name("areas")
			out.beginArray()
			value.areas.forEach { out.value(it.jsonName) }
			out.endArray()
			out.name("region")
			out.value(value.region.jsonName)
			out.name("time")
			out.value(value.time.jsonName)
			out.endObject()
		}

		override fun read(`in`: JsonReader): SpawnParameters {
			val json = `in`.popObject()

			val areas = if ("area" in json) {
				if (json["area"].asString == "all")
					SpawnArea.ALL
				else
					setOf(SpawnArea.entries.valueOf(json["area"].asString))
			} else if ("areas" in json) {
				json["areas"].asJsonArray.stream().map { SpawnArea.entries.valueOf(it.asString) }.collect(Collectors.toCollection { EnumSet.noneOf(SpawnArea::class.java) })
			} else {
				setOf()
			}

			val region = SpawnRegion.entries.valueOf(json["region"].asString)
			val time = SpawnTime.entries.valueOf(json["time"].asString)
			return SpawnParameters(areas, region, time)
		}
	}
}
