package ru.dbotthepony.kstarbound.world.entities.behavior

import org.apache.logging.log4j.LogManager
import org.classdump.luna.Table
import org.classdump.luna.exec.CallPausedException
import org.classdump.luna.lib.CoroutineLib
import org.classdump.luna.runtime.Coroutine
import ru.dbotthepony.kstarbound.defs.actor.behavior.NodeOutput
import ru.dbotthepony.kstarbound.defs.actor.behavior.NodeParameter
import ru.dbotthepony.kstarbound.lua.userdata.BehaviorState

class ActionNode(val name: String, val parameters: Map<String, NodeParameter>, val outputs: Map<String, NodeOutput>) : AbstractBehaviorNode() {
	private var coroutine: Coroutine? = null
	private val nodeID = BehaviorState.NODE_GARBAGE_INDEX.getAndIncrement()

	override fun run(delta: Double, state: BehaviorState): Status {
		var coroutine = coroutine
		var firstTime = false

		if (coroutine == null) {
			firstTime = true

			val fn = state.functions[name] ?: throw RuntimeException("How? $name")

			coroutine = state.lua.call(CoroutineLib.create(), fn)[0] as Coroutine
		}

		try {
			val result = if (firstTime) {
				val parameters = state.blackboard.parameters(parameters, this)
				state.lua.call(CoroutineLib.resume(), coroutine, parameters, state.blackboard, nodeID, delta)
			} else {
				state.lua.call(CoroutineLib.resume(), coroutine, delta)
			}

			val coroutineStatus = result[0] as Boolean

			if (!coroutineStatus) {
				LOGGER.warn("Behavior ActionNode '$name' failed: ${result[1]}")
				return Status.FAILURE
			}

			if (result.size == 1) {
				return Status.RUNNING
			}

			val nodeStatus = result.getOrNull(1) as? Boolean ?: false
			val nodeExtra = result.getOrNull(2) as? Table

			if (nodeExtra != null) {
				state.blackboard.setOutput(this, nodeExtra)
			}

			if (!nodeStatus) {
				return Status.FAILURE
			} else {
				return Status.SUCCESS
			}
		} catch (err: CallPausedException) {
			LOGGER.error("Behavior ActionNode '$name' called blocking code, which initiated pause. This is not supported.")
			return Status.FAILURE
		}
	}

	override fun reset() {
		coroutine = null
	}

	override fun toString(): String {
		return "ActionNode[$name, parameters=$parameters, outputs=$outputs, coroutine=$coroutine]"
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}
